/* Common constants */
const Double_t u = 931.49432/1000.; // GeV
const Double_t c = 299.7924; // speed of light, mm/ns 
const Double_t pi = 4.*atan(1.);
const Double_t d2r = pi/180.;
const Double_t dummy = -1000.;
//////////////////////////

/* Event information */
//Long64_t ntot = 2.e9; // number of events
//Long64_t ntot = 1.e8; // number of events
const Long64_t ntot = 2.e5; // number of events
Char_t label[50] = "c11li6n14pd"; // 
//Char_t simnum[10] = "sim001"; // No beam spread, detector resolutions (in energy, space)
//Char_t simnum[10] = "sim002"; // only beam (energy, spacial, PPAC resolution) spread
Char_t simnum[10] = "sim003"; // only SSD energy resolution */
/* Char_t simnum[10] = "sim004"; // only SSD strip width */
/* Char_t simnum[10] = "sim005"; // including beam spread, SSD resolutions */
/* Char_t simnum[10] = "sim006"; // Optimization of ... */
//////////////////////////


/* particles information */
Int_t z1 = 1;
float mh= 1.00782503;
float mn= 1.00866492; 
float md= 2.01410178; 
//float mt= 3.01604928; 
Int_t z2 = 2;
float ma= 4.00260325;
Int_t z3 = 3;
float m6li= 6.01512279;
Int_t z6 = 6;
float m11c = 11.01143361;
Int_t z7 = 7;
float m14n = 14.00307400;
const Int_t Nexci = 1;
Double_t exci[Nexci] = {0.}; // in MeV, 14N
//Double_t exci[Nexci] = {0., 2.312798, 3.94810, 4.9151}; // in MeV, 14N
Int_t zprj = z6;
float mprj = m11c;
Int_t ztrg = z3;
float mtrg = m6li;
Int_t zres = z7;
float mprt = ma;
float mres[Nexci], q2[Nexci];
float mres_gs = m14n;
Int_t zrec = z1;
float mrec = mh; 
Int_t zspc = z1;
float mspc = md; 
Int_t atom_num[3] = {zres, zrec, zspc} ; // atomic numbers
//////////////////////////


/* parameters for eoldz */
float Eaftpu, Eaftpu_beam;
float Eaft, Eaft_beam; 
//Char_t al[34] = "al";
//Char_t ch2[34] = "POLYETHYLENE";
//Char_t cd2[34] = "cd2";
Char_t lif[34] = "lif";
Char_t he[34] = "he";
Char_t si[34] = "si";
Int_t unit_pressure = 1; // Torr
// float pressure0 = 0.; //dummy
float pressure = 0.; //dummy
// float dpressure = 0.; // sigma
//float dpressure = 1.; // sigma
//float pressure, temperature; // sigma
Int_t unit_temperature = 1; // K, for srho_gas
//float temperature0 = 293.;
//float temperature0 = 299.;
float temperature = 299.;
//float dtemperature = 0.; // sigma
//float dtemperature = 1.; // sigma
Int_t unit_mm = 1; // in mm
Int_t unit_mgpcm2 = 2; // in mg/cm^2
float thk_tar = 0.; // Target thickness in mg/cm^2 // sim001 sim003
//float thk_tar = 0.15; // Target thickness in mg/cm^2 // sim002
float r_sphere = 250.; // radius of the TGeoVolume in mm
//float dEbin = 0.05; // energy step for effective thickness calculation
float Ebfrpu, Ebfrpu_beam; // Beam energy E in MeV/u after energy loss in matter
float Ebfr, Ebfr_beam; // Beam energy E in MeV after energy loss in matter
//////////////////////////


/* Beam condition */
// 24.4 MeV --(2xPPACs 19 um Mylar)--(1/2 LiF target 75 ug/cm^2)-- 11 MeV
// 24.4 * 2 *10/16/100 * 0.68(1 sigma) = 0.21 MeV 
// Monte Carlo reaction depth in target
// |---x---| Reference: middle -> E.beam_mid = (eprj_mid +/- deprj)*1000.
// |---|-x-| Practice: -1/2 thk_tar -- +1/2 thk_tar -> E.beam_MC
// -1------1 
Double_t eprj_mid = 11./1000.; // Mean beam energy at entrance
Double_t deprj = 0./1000.; // Beam energy spread (GeV) // sim001 sim003
//Double_t deprj = 0.21/1000.; // Beam energy spread (GeV) // sim002
//Double_t depe2 = 0.; // Relative energy resolution DE/E @5MeV sim001, sim002
Double_t depe2 = 0.01; // Relative energy resolution DE/E @5MeV sim003
Double_t xtar0 = 0.; // mm
Double_t ytar0 = 0.; // mm
Double_t xtar0_sgm = 0.; // mm sim001 sim003
Double_t ytar0_sgm = 0.; // mm  sim001 sim003
/* Double_t xtar0_sgm = 5.; // sim002 */
/* Double_t ytar0_sgm = 10.;  // sim002 */
//
Double_t za = - 600.; // mm, PPAC a z position
Double_t zb = za + 500.; // mm, PPAC b z position
Double_t sgm_ppac_a = 0.; // mm, PPAC a intrinsic resolution // sim001 sim003
Double_t sgm_ppac_b = 0.; // mm, PPAC b intrinsic resolution // sim001 sim003
/* Double_t sgm_ppac_a = 0.5; // mm, PPAC a intrinsic resolution sim002 */
/* Double_t sgm_ppac_b = 0.5; // mm, PPAC b intrinsic resolution sim002 */
//////////////////////////


/* Detector arrangement */
const Int_t ndet = 6; 
//Double_t thdet[ndet] = {-56., -34., -12., 12., 34., 56.}; // in deg
Double_t thdet[ndet] = {-60., -40., -12., 12., 40., 60.}; // in deg
Double_t ldet[ndet] = {200., 200., 200., 200., 200., 200.};
Double_t xdet_off[ndet] = {0., 0., 0., 0., 0., 0.};
Double_t zdet_off[ndet] = {0., 0., 0., 0., 0., 0.};
// float int_det = 10.; // interval between DE and E layers (mm)
// float de_thick = 0.02; // DE thickness (mm)
Double_t psd_half[ndet] = {25., 25., 25., 25., 25., 25.}; 
Int_t n_strip[ndet] = {0}; // sim001 sim002 sim003
//Int_t n_strip[ndet] = {16, 16, 16, 16, 16, 16}; // sim003
Double_t strip_half[ndet];
//Double_t dth[ndet]; // 03-04-2018
//////////////////////////

//Double_t dt = 1.; // TOF resolution in nsec.


