#include <TGeoManager.h>
#include <TGeometry.h>
#include <TNode.h>
#include <TGeoVolume.h>
#include <TGeoMatrix.h>
#include <TGeoMedium.h>
#include <TGeoMaterial.h>
#include <TGeoNode.h>
#include <TMaterial.h>
#include <TMixture.h>
#include <TShape.h>
#include <TString.h>
#include <Riostream.h>
#include <TFile.h>
#include <TCanvas.h>
#include <TH2F.h>
#include <TMath.h>
#include <TLorentzVector.h>
#include <TSystem.h>
#include <stdlib.h>
#include <TROOT.h>
#include <TGenPhaseSpace.h>
#include <TStyle.h>
#include "TGeoPatternFinder.h"
#include "TSystem.h"
#include <TGeoNavigator.h>
#include <iostream>
#include <cstdlib>
#include <fstream>
#include <cmath>
#include "TApplication.h"
#include "TF1.h"
#include "TGraph.h"
#include "TGraphErrors.h"
#include "TH1.h"
#include "TLegend.h"
#include "TLegendEntry.h"
#include "TColor.h"
#include <TRandom3.h>
#include <TTree.h>
#include <time.h> // for the seed of the dandom number
#include <TView3D.h>

#include "Math/Interpolator.h"
//#include "Math/Polynomial.h"
#include "Math/InterpolationTypes.h"
//#include "Math/Types.h"

#include "prm_reac.C" // parameter file <--

using namespace std;

extern "C" {
  void enewzsub_(int *z1, float *m1, float *bfr_ene, char matter1[33], 
  		 int *unit_pressure, float *pressure, float *temperature, 
  		 int *unit_thick, float *thick1, float *aft_ene);
  void eoldzsub_(int *z1, float *m1, float *aft_ene, char matter1[33], 
  		 int *unit_pressure, float *pressure, float *temperature, 
  		 int *unit_thick, float *thick1, float *bfr_ene);
  void rangezsub_(int *z1, float *m1, float *bfr_ene, char matter1[33], 
		  float *range);
  void srhogas_(float *pressure, float *temperature, int *unit_pressure,
		int *unit_temperature, float *mol_weight, 
		float *density);
}

// effective thickness //
// Double_t eff_thk(Double_t x ){ // beam energy in GeV
//   for(Int_t ichar=0;ichar<33;ichar++){ 
//     if(he[ichar]=='\0'){
//       he[ichar]=' ';
//       he[ichar+1]='\0';
//     }
//   }
  
//   //float dxpu = dEbin*(m26si + ma)/m26si/ma; // energy bin size in AMeV
//   float dxpu = dEbin*(mprj + mtrg)/mprj/mtrg; // energy bin size in AMeV
//   float xpu;
//   float range1, range2, drange;

//   xpu = x*1000./mprj + dxpu/2.;
//   rangezsub_(&zprj,&mprj,&xpu,he,&range1);
//   xpu = x*1000./mprj - dxpu/2.;
//   rangezsub_(&zprj,&mprj,&xpu,he,&range2);

//   drange = fabs(range1 - range2);
//   //cout << x*1000. << " " << range2 << " " << range1 << " " << drange << endl;
  
//   return(drange);
// }

Double_t dtof(Double_t *x, Double_t *p){
  for(Int_t ichar=0;ichar<33;ichar++){ 
    if(he[ichar]=='\0'){
      he[ichar]=' ';
      he[ichar+1]='\0';
    }
  }

  int z = int(p[0]);
  float m = float(p[1]);
  float ebpu = float(p[2]);
  float l = float(x[0]);
  // to be integrated: dtof = dx/v(x)
  float eapu; // MeV
  enewzsub_(&z, &m, &ebpu, he, &unit_pressure, &pressure, 
	    &temperature, &unit_mm, &l, &eapu);
  // cout << z << " " << m << " " << ebpu << " " << l << " " << eapu << endl;
  // getchar();
  return sqrt(u*1000./2./eapu)/c;
}

Double_t Elab2cm(Double_t Elab, Double_t thlab,
		 Double_t mres_ex, Double_t q2ex){
  Double_t A = mtrg*(mprj + mtrg)/mprj/mrec*Elab;
  Double_t B = mtrg*mres_ex/mprj/mrec;
  Double_t D = 1. - B;
  Double_t C = 2.*pow(cos(thlab*d2r),2) - D; // 1. - B;
  // cout << A <<" "<< B  <<" "<< C <<" "<< D << endl;
  //cout << q2 <<" ";
  
  Double_t Ecm;
  //cout << q2ex << endl;
  if(q2ex > 5.e-7){
    Ecm= ( A*C + B*q2ex*D*1000. - 2.*cos(thlab*d2r)
	   *sqrt( A*(A*(B - pow(sin(thlab*d2r),2)) + B*q2ex*D*1000.) ) )
      /D/D;
  }else{ // Q = 0, elastic scattering
    Ecm = (mtrg + mprj)/4./mprj/pow(cos(thlab*d2r),2)*Elab;
  }
  // Double_t Ecm = ( A*C + B*q2ex*D - 2.*cos(thlab*d2r)
  // 		   *sqrt( A*(A*(B - pow(sin(thlab*d2r),2)) + B*q2ex*D) ) )/D/D;
  //cout << Elab <<" "<< thlab<< " " <<Ecm <<endl ;
  //getchar();

  // if(isnan(abs(Ecm))){
  //   cout << thlab << " " << A << " " << B << " " << C << " " << D << " "
  //     //<< A*(B - pow(sin(thlab*d2r),2)) << " " << B*q2ex*D*1000.
  // 	 << pow(sin(thlab*d2r),2) << " " << B - pow(sin(thlab*d2r),2)
  // 	 << endl;
  //   getchar();
  // }
  return Ecm;
}

Double_t thlab2cm(Double_t Ecm, Double_t thlab,
		  Double_t mres_ex, Double_t q2ex){
  Double_t g = sqrt(mprj*mrec*Ecm/mtrg/mres_ex/(Ecm + q2ex));
  Double_t thcm; // = atan( (tan(thlab*d2r)
  //	 *(1. + g*sqrt(1. +(1. -g*g)*pow(tan(thlab*d2r),2))))
  //			/ (1 - g*g*pow(tan(thlab*d2r),2)) )/d2r;
  // Double_t thcm_p = atan( (tan(thlab*d2r)
  // 			   *(1. + g*sqrt(1. +(1. -g*g)*pow(tan(thlab*d2r),2))))
  // 			  / (1 - g*g*pow(tan(thlab*d2r),2)) )/d2r;
  // Double_t thcm_m = atan( (tan(thlab*d2r)
  // 			   *(1. - g*sqrt(1. +(1. -g*g)*pow(tan(thlab*d2r),2))))
  // 			  / (1 - g*g*pow(tan(thlab*d2r),2)) )/d2r;
  //if(thcm < 0.) thcm += 180.;
  //cout << g*sqrt(1. +(1. -g*g)*pow(tan(thlab*d2r),2)) << endl;
  //if(1. - g*sqrt(1. +(1. -g*g)*pow(tan(thlab*d2r),2)) < 0.){
  // if(1. - g*g*pow(tan(thlab*d2r),2) > 0.){
  thcm = atan( (tan(thlab*d2r)
		*(1. + g*sqrt(1. +(1. -g*g)*pow(tan(thlab*d2r),2))))
	       / (1 - g*g*pow(tan(thlab*d2r),2)) )/d2r;

  if(thcm < 0)
    thcm += 180.;
  // }else{
  //   thcm = atan( (tan(thlab*d2r)
  // 		  *(1. + g*sqrt(1. +(1. -g*g)*pow(tan(thlab*d2r),2))))
  // 		 / (1. - g*g*pow(tan(thlab*d2r),2)) )/d2r;
  // }
  // cout << thlab << " "
  //      << 1. + g*sqrt(1. +(1. -g*g)*pow(tan(thlab*d2r),2)) << " "
  //      << 1. - g*sqrt(1. +(1. -g*g)*pow(tan(thlab*d2r),2)) << " "
  //      << (1. - g*g*pow(tan(thlab*d2r),2)) << " "
  //      << thcm <<  endl;

  // cout << sin(thcm_p*d2r) << " "
  //   	 << cos(thcm_p*d2r) << " "
  // 	 << sin(thcm_m*d2r) << " "
  //   	 << cos(thcm_m*d2r) << " "
  // 	 << sin(thcm_p*d2r)/(cos(thcm_p*d2r)+g) << " "
  // 	 << sin(thcm_m*d2r)/(cos(thcm_m*d2r)+g) << " "
  // 	 << tan(thlab*d2r) << endl;
    
  return thcm;
}

Double_t ebeam2thk(Double_t Ebeam, Double_t Ebeam_ini){ //at z & entrance
  for(Int_t ichar=0;ichar<33;ichar++){ 
    if(he[ichar]=='\0'){
      he[ichar]=' ';
      he[ichar+1]='\0';
    }
  }
  
  float xpu;
  float range1, range2, drange;

  xpu = Ebeam/mprj; // MeV/u
  rangezsub_(&zprj,&mprj,&xpu,he,&range1);
  xpu = Ebeam_ini/mprj;// MeV/u
  rangezsub_(&zprj,&mprj,&xpu,he,&range2);

  drange = range2 - range1; // mg/cm^2
  
  return(drange);
}

Double_t sgm_ppac_ab(Double_t z){
  
  //Double_t sgm = sgm_ppac*sqrt(pow((zab-z)/zab,2) + pow(z/zab,2));
  // Double_t sgm = sgm_ppac*sqrt(pow((zb-z)/(zb-za),2)
  // 			       + pow((za-z)/(zb-za),2));
  Double_t sgm = sqrt(pow((zb-z)/(zb-za),2)*sgm_ppac_a
		      + pow((za-z)/(zb-za),2)*sgm_ppac_b);
  return sgm;
};



int main() {
  Double_t strip_half = psd_half/n_strip; //13-10-2021
 
  
  TApplication theApp("App", NULL, NULL);

  //ROOT::Math::IntegratorOneDimOptions::SetDefaultIntegrator("Gauss");
  
  // For enewz
  for(Int_t ichar=0;ichar<33;ichar++){ 
    if(he[ichar]=='\0'){
      he[ichar]=' ';
      he[ichar+1]='\0';
    }
    if(si[ichar]=='\0'){
      si[ichar]=' ';
      si[ichar+1]='\0';
    }
  }
  //float density = 5.4743e-05; // of the target gas (g/cm^3)
  // srhogas_(&pressure, &temperature, &unit_pressure,
  // 	   &unit_temperature, &ma, 
  // 	   &density);
  // cout << density << endl; getchar();
  
  // For TOF integration
  TF1 *fint = new TF1("fint",dtof,0.,ltar,3);
  
  /* // NON-SMOKER data */
  /* const Int_t nhf = 90; // no. of data */
  /* Double_t ehf[nhf], notused, cshf[nhf]; */
  /* ifstream datahf("../../../nonsmoker_cs.txt");  */
  /* for(Int_t i=0; i<nhf; i++ ){ // i=1 to avoid the 1st row */
  /*   datahf >> ehf[i] >> notused >> notused >> cshf[i]; */
  /*   //cout << i << " " << ehf[i] <<" "<< cshf[i] << endl; */
  /* } */
  /* //getchar(); */
  /* datahf.close(); */
  /* ROOT::Math::Interpolator interhf(nhf, ROOT::Math::Interpolation::kLINEAR); */
  /* interhf.SetData(nhf, ehf, cshf); */

  Char_t fname[150];
  sprintf(fname,"root/%s.root",label);
  cout << "Output file name: " << fname << endl;
  cout << "OK to run?" << endl; getchar();
  TFile *fl2 = new TFile(fname,"RECREATE");
  TTree* sim = new TTree("sim","");

  // variables for tree
  typedef struct{
    Double_t beam, beam0, beam0_resol;
    Double_t lab[3], lab_det[2], lab_det_resol[2], lab_resol[2];
    Double_t lab_DE[2], lab_DE_resol[2], lab_RES[2], lab_RES_resol[2];
    Double_t rel[3], rel_resol[3], cm, cm_z, cm_resol, cm_z_resol, cm_z0, cm_dev;
  } ENERGY;
  ENERGY E;
  sim->Branch("E",&E,"beam/D:beam0:beam0_resol:"
	      "lab[3]:lab_det[2]:lab_det_resol[2]:lab_resol[2]:"
	      "lab_DE[2]:lab_DE_resol[2]:lab_RES[2]:lab_RES_resol[2]:"
	      "rel[3]:rel_resol[3]:cm:cm_z:cm_resol:cm_z_resol:cm_z0:cm_dev");
  
  typedef struct{
    Double_t lab[3], lab_resol[2], cm, cm_resol, cm_dev, cm_resol_nr;
  } THETA;
  THETA th;
  sim->Branch("th",&th,"lab[3]/D:lab_resol[2]:cm:cm_resol:cm_dev:cm_resol_nr");

  typedef struct {
    Double_t lab[3], lab_resol[2]; 
  } PHI;
  PHI phi;
  sim->Branch("phi",&phi,"lab[3]/D:lab_resol[2]");

  typedef struct {
    Double_t spc; 
  } MOMENTUM;
  MOMENTUM p;
  sim->Branch("p",&p,"spc/D");

  typedef struct {
    Double_t xtar, ytar, rtar;
    Double_t ztar;
    Double_t xtar_resol, ytar_resol, ztar0, ztar_resol;
    Double_t ldetprt[2], ldetprt_resol[2];
    Double_t xhit[2][ndet], yhit[2][ndet], xhit_resol[2][ndet], yhit_resol[2][ndet];  
  } POSITION;
  POSITION pos;
  sim->Branch("pos",&pos,
	      Form("xtar/D:ytar:rtar:ztar:"
		   "xtar_resol:ytar_resol:ztar0:ztar_resol:"
		   "ldetprt[2]:ldetprt_resol[2]:"
		   "xhit[2][%d]:yhit[2][%d]:"
		   "xhit_resol[2][%d]:yhit_resol[2][%d]",
		   ndet,ndet,ndet,ndet)
	      );

  typedef struct {
    //Double_t de, e, beam, rec, dee, de_resol, e_resol, beam_resol, rec_resol;
    Double_t beam, beam_resol, rec, rec_resol, de, de_resol;
  } TOF;
  TOF tof;
  sim->Branch("tof",&tof,"beam/D:beam_resol:rec:rec_resol:de:de_resol");
  
  typedef struct{
    Int_t idet[2]; // for particle 1&2
    Bool_t any;
    //Int_t any,idet;
  } HIT;
  HIT hit;
  //sim->Branch("hit",&hit,"any/O:idet/I");
  sim->Branch("hit",&hit,"idet[2]/I:any/O");
  // Bool_t any;
  // Int_t idet;
  // sim->Branch("any",&any,"any/O");
  // sim->Branch("idet",&idet,"idet/I");

  typedef struct{
    Double_t prs, prs0, tmp, tmp0;
  } GAS;
  GAS gas;
  sim->Branch("gas",&gas,"prs/D:prs0:tmp:tmp0");
  
  typedef struct {
    Double_t evt; //, efth, hf, yld;
  } WEIGHT;
  WEIGHT w;
  sim->Branch("w",&w,"evt/D"); //:efth:hf:yld");
  //
  Int_t iex;
  sim->Branch("iex",&iex,"iex/I");  
  for(Int_t iexci=0; iexci<Nexci; iexci++){ // define mass and q value for excited states
    mres[iexci] = mres_gs + exci[iexci]/u/1000.; // (a,p) 
    q2[iexci] = (mprj + mtrg - mres[iexci] - mrec)*u; 
    //cout << i <<" " <<mres[i] <<" "<< q2[i] << endl;
  }
  //getchar();

  //////////////////////////////////////

  gStyle->SetPalette(1);

  gSystem->Load("libPhysics.so");

  gSystem->Load("libGeom.so");

  TGeoManager *sc = new TGeoManager("scattcham","Scattering Chamber");
 
  TGeoMaterial *matVacuum = new TGeoMaterial("Vacuum",0,0,0);
  TGeoMaterial *matSi = new TGeoMaterial("Si",28.086,14,2.321);
 
  TGeoMedium *Vacuum = new TGeoMedium("Vacuum",1,matVacuum);
  TGeoMedium *Si = new TGeoMedium("Root Material",2,matSi);

  TGeoVolume *top = sc->MakeSphere("TOP",Vacuum,0.,ltar/2.,0.,180.,0.,360.); //mm
  sc->SetTopVolume(top);
  
  TGeoVolume *psd = sc->MakeBox("PSD",Si,psd_half,psd_half,0.5); // in mm // 03-04-2018
  // (X,Y,Z,rotation angle theta originating at (X,Y,Z) )
  // tan(theta) = X/Z
  // TGeoCombiTrans *pos1 = new TGeoCombiTrans(12.28,0,16.29,new TGeoRotation("rot1",90,37,0));
  // TGeoCombiTrans *pos2 = new TGeoCombiTrans(-4.788,0,13.16,new TGeoRotation("rot2",90,-20,0));

  TGeoCombiTrans *posdet[ndet];
  Char_t rotname[10];
  TVector3 vdet[ndet], voff[ndet];

  // Detectors 
  for(Int_t i=0;i<ndet;i++){
    vdet[i].SetXYZ( ldet[i]*sin(d2r*thdet[i])+xdet_off[i],
		    0,ldet[i]*cos(d2r*thdet[i])+zdet_off[i] );
    //dth[i] = atan(psd_half/ldet[i])/d2r;
    sprintf(rotname,"rot%d",i);
    posdet[i] = new 
      // Rotate detectors
      //TGeoCombiTrans(ldet[i]*sin(d2r*thdet[i]),0,ldet[i]*cos(d2r*thdet[i]),
      // TGeoCombiTrans(ldet[i]*sin(d2r*thdet[i])+xdet_off[i],
      // 		     0, ldet[i]*cos(d2r*thdet[i])+zdet_off[i],
      // 		     new TGeoRotation(rotname,90,thdet[i],0));
      TGeoCombiTrans(vdet[i].X(),vdet[i].Y(),vdet[i].Z(),
      		     new TGeoRotation(rotname,90,thdet[i],0));
      // Put detectors at coordinates 
      // TGeoCombiTrans(xdet[i],0.,zdet[i], new TGeoRotation(rotname,90,rotdet[i],0));
    top->AddNode(psd,i,posdet[i]);
    voff[i].SetXYZ(xdet_off[i],0.,zdet_off[i]);
  }
  
  gGeoManager->CloseGeometry();
  top->SetLineColor(kRed);
  gGeoManager->SetTopVisible();
  ///////////////////////////////////////////////////////////////
  
  // For theta
  
  TVector3 vhit[2][ndet];
  TVector3 vhit_resol[2];
  TVector3 vdetprt[2], vdetprt_resol[2]; // paticle #2

  TLorentzVector plrz[3];
  TVector3 pmom[3]; // particle #1, #2
  TVector3 pdir[2];
  TLorentzVector plrzsum[3]; // #1+#2, #2+#3, #3+#1
  TVector3 v[3];
  
  //TCanvas *c0 = new TCanvas("c0","c0",10,10,800,800);
  TCanvas *c0 = new TCanvas("c0","c0",1210,10,800,800);
  //gPad->SetGrid(1);
  top->Draw();
  //c0->SaveAs("sim_geom_prova_o16.ps");
  //return 0;
  c0->Update();
  gSystem->ProcessEvents();
  // getchar();
  
  TLorentzVector target(0.0, 0.0, 0.0, u*mtrg); // px, py, pz, e
  // TLorentzVector beam(0.0, 0.0, pp, u*mprj+eprj0); // px, py, pz, e
  // TLorentzVector W = beam + target; // coumpound of beam + target
  
  //(Momentum, Energy units are Gev/C, GeV)
  //Double_t mass[3] = { u*mres, u*mrec, u*m3} ; // masses of 7Li,p,p
  //Double_t mass[2] = { u*mres, u*mrec} ; // masses of 29P,p
  
  TGenPhaseSpace event;
  // event.SetDecay(W, 3, mass); // W: beam + target -> 3-body decay: mres+mrec+m3
  
  //Int_t seed = 12345;
  Int_t seed = time(NULL);
  TRandom3* ran = new TRandom3(seed); //

  Long64_t i = 0;
  Long64_t geve=0;
  //Long64_t nentries = 1000000;
  //Long64_t nentries = 1.e7;
  Long64_t nstep = ntot/4321;
  //while (geve<nentries) { // number of accepted events
  printf("%6.2f %% done, %6.2f %% valid \r",
  	 float(i)/float(ntot)*100.,
  	 float(geve)/float(i)*100.);     

  // TLorentzVector beam0(0.0, 0.0, sqrt(2.*u*mprj*eprj0), u*mprj+eprj0); // px, py, pz, e
  // TLorentzVector W0 = beam0 + target; // coumpound of beam + target
  // TVector3 v0_0 = pow(u*mprj,-1)/beam0.Gamma() *(beam0.Vect()); // velocity 
  // Double_t weight_total = 0;
  // Double_t weight_in = 0;
  
  for(Int_t ievt=0; ievt<ntot; ievt++){
    // Initialization of structures //
    E = {dummy};
    th = {0};
    p = {dummy};
    tof = {dummy};
    hit.idet[0] = hit.idet[1] = -1;
    hit.any = false;
    pos = {dummy};
    pos.ztar = pos.ztar0 = pos.ztar_resol = 0.; // z = 0 is the target position
    w = {0};
    iex = -1;
    
    // Definition of the excited state in 13N
    iex = ran->Rndm()*Nexci; // 0-1 * Nexci
    Double_t mass[3] = { u*mres[iex], u*mrec, u*mspc} ; // masses of 11C, alpha, n
    
    // Definition of beam energy
    //eprj = eprj0 + ran->Gaus(0.,deprj); // GeV
    //eprj = ran->Gaus(eprj0,deprj); // GeV
    //eprj = eprj0; // Beam energy at entrance, GeV, 10-nov-2021
    eprjini = ran->Gaus(eprj0ini,deprj); // ! for now deprj is constant
    eprj = eprjini; // reaction position
    
    // Beam, compound, decay
    TLorentzVector beam(0.0, 0.0, sqrt(2.*u*mprj*eprj), u*mprj+eprj); // px, py, pz, e
    TLorentzVector W = beam + target; // coumpound of beam + target
    event.SetDecay(W, 3, mass); // W: beam + target -> 3-body decay: mres+mrec+m3
    w.evt = event.Generate();
    
    // to show the process //
    if(ievt%nstep == 0) 
      printf("%6.2f %% done, %6.2f %% valid \r",
    	     float(ievt)/float(ntot)*100.,
    	     float(geve)/float(ievt)*100.);     
    /////////////////////////
    
    // Decay kinematics
    for(Int_t iprt=0; iprt<3; iprt++){
      plrz[iprt] = *event.GetDecay(iprt); // lorentz vector      
      pmom[iprt] = plrz[iprt].Vect(); // momenta
      v[iprt] = pow(mass[iprt],-1)/plrz[iprt].Gamma() *pmom[iprt];
      th.lab[iprt] = (plrz[iprt].Theta())/d2r;
      phi.lab[iprt] = (pmom[iprt].Phi())/d2r;
      plrzsum[iprt] = plrz[iprt] + plrz[(iprt+1)%3]; // (iprt+1)%3 = 1, 2, 0
    }
    for(Int_t iprt=0; iprt<3; iprt++){
      if(iprt < 2){
	E.lab[iprt] = (plrz[iprt].Energy()-mass[iprt])*1000.; // MeV
      }else{ // iprt == 2
	TVector3 pp3vec = beam.Vect() - pmom[0] - pmom[1]; // momentum conservation
	E.lab[2] = pp3vec.Mag2()/2./mass[2]*1000.;
      }
      E.rel[iprt] = (plrzsum[iprt].M() - (mass[iprt] + mass[(iprt+1)%3]))*1000.; // MeV
    }
    E.cm = E.rel[0] - q2[iex]*1000.; // MeV
    
    TVector3 vprj = pow(u*mprj,-1)/beam.Gamma() *(beam.Vect()); // velocity 
    TLorentzVector ptran = - plrz[2]; // 4-momentum of the transferred particle 
    TVector3 vtran = 1./sqrt(pow(mtrg*u - plrz[2].Energy(),2) + pmom[2].Mag2())
      /ptran.Gamma() *ptran.Vect(); // velocity, See Slaus et al
    TVector3 ki = vprj; - vtran; // target breakup 13-11-2017
    TVector3 kf = v[0] - v[1]; // Final relative velocity (1<->2); v1 is beamlike (heavy ion)
    th.cm = (ki.Angle(kf))/d2r;

    Double_t scp;
    if( cos(pmom[2].Phi())>0 ) // x>0 direction
      scp=1;
    else if ( cos(pmom[2].Phi())<0 ) // x<0 direction
      scp=-1.;
    else 
      scp=0.;
    p.spc = pmom[2].Mag()*scp*1000.; // in MeV/c target breakup
    
    // Beam hit position
    pos.xtar = ran->Gaus(xtar0,xtar0_sgm);
    pos.ytar = ran->Gaus(ytar0,ytar0_sgm);
    pos.rtar = sqrt(pos.xtar*pos.xtar + pos.ytar*pos.ytar);
    //TVector3 vtar(xtar,ytar,0); // vector from origin to target pos.
    TVector3 vreac(pos.xtar,pos.ytar,pos.ztar); // vector to the reaction point.

    // Tracking decaying particles
    /// Particle 1: residual heavy ion
    /// Particle 2: recoil light particle
    /// Particle 3: spectator
    Int_t hitdet[2];
    Double_t xx, yy, zz;
    Double_t snext;
    Bool_t istate[2] = {false, false};
    Int_t idnode[2];
    const Char_t *path;
    Char_t sdet[2];
    for(Int_t iprt = 0; iprt<2; iprt++){
      pdir[iprt] = pmom[iprt].Unit(); // represents the direction
      xx = pdir[iprt](0);
      yy = pdir[iprt](1);
      zz = pdir[iprt](2);
      gGeoManager->SetCurrentPoint(pos.xtar,pos.ytar,pos.ztar);
      gGeoManager->SetCurrentDirection(xx,yy,zz);
      gGeoManager->GetCurrentNode();
      gGeoManager->FindNode();
      gGeoManager->FindNextBoundary(radius);
      snext = gGeoManager->GetStep();
      gGeoManager->Step();
      istate[iprt] = gGeoManager->IsStepEntering();
      idnode[iprt] = gGeoManager->GetCurrentNodeId();
      //Double_t safety2 = gGeoManager->GetSafeDistance();
      /* cout << iprt << " " */
      /* 	   << istate[iprt] <<" "<< snext[iprt] <<" " << idnode[iprt] << endl; */
      //const Char_t *path1
      path = gGeoManager->GetPath(); // /TOP_1/PSD_3 etc.
      strncpy(sdet, path+11, 1); // extract the 12th character from path
      sdet[1] = '\0';
      hitdet[iprt] = atoi(sdet);
      vdetprt[iprt].SetXYZ(snext*xx,snext*yy,snext*zz);
      pos.ldetprt[iprt] = snext;      
    }
    
    // If particles hit any detectors
    if(istate[0] || istate[1]){
      hit.any = true;
      /* cout << "any" << endl; */
    }
    // If both particle 1 & 2 hit different detectors respectively
    if(istate[0] && istate[1] && idnode[0]!=idnode[1]){	      
      for(Int_t iprt = 0; iprt<2; iprt++){
	hit.idet[iprt] = hitdet[iprt];
	// Integerization: no. of the hit detector // Warning: when no hit, it returns 0!	
	//vhit[hit.idet] = vtar + vdetprt - vdet[hit.idet] - voff[hit.idet];
	vhit[iprt][hit.idet[iprt]] = vdetprt[iprt] + vreac - vdet[hit.idet[iprt]];
	//vhit[hit.idet] -= voff[hit.idet];
	vhit[iprt][hit.idet[iprt]].RotateY(-d2r*thdet[hit.idet[iprt]]);
	pos.xhit[iprt][hit.idet[iprt]] = vhit[iprt][hit.idet[iprt]].X();
	pos.yhit[iprt][hit.idet[iprt]] = vhit[iprt][hit.idet[iprt]].Y();
      } // for iprt
      //cout << hit.idet[1] << " " << idnode[1] << endl;
      /* float fldetprt = float(pos.ldetprt); */
      /* Ebfrpu = E.lab[1]/mrec; // MeV */
      /* enewzsub_(&zrec, &mrec, &Ebfrpu, he, &unit_pressure, &pressure,  */
      /* 	  &temperature, &unit_mm, &fldetprt, &Eaftpu); */
      /* E.lab_det[1] = Eaftpu*mrec; // MeV */
      
      ///////////////////////////
      // Study with resolution //
      ///////////////////////////
      /* Double_t sgm_track = sgm_ppac_ab(pos.ztar); */
      /* pos.xtar_resol = ran->Gaus(pos.xtar,sgm_track); */
      /* pos.ytar_resol = ran->Gaus(pos.ytar,sgm_track); */
      
      
      /* //xhit_resol[hit.idet] = ran->Gaus(xhit[hit.idet],dx_det); // ! */
      /* pos.xhit_resol[hit.idet] = ran->Uniform(pos.xhit[hit.idet]-strip_half, */
      /* 				    pos.xhit[hit.idet]+strip_half); // for strip */
      /* //yhit_resol[hit.idet] = ran->Gaus(yhit[hit.idet],dy_det); // ! */
      /* pos.yhit_resol[hit.idet] = ran->Uniform(pos.yhit[hit.idet]-strip_half, */
      /* 				    pos.yhit[hit.idet]+strip_half); // for strip   */
      /* vhit_resol.SetXYZ( pos.xhit_resol[hit.idet], pos.yhit_resol[hit.idet], 0 ); //vector on det. surface */
      /* vhit_resol.RotateY(d2r*thdet[hit.idet]); // Rotate from z axis to det. angle */
      /* //vhit_resol += voff[hit.idet]; */
      /* //vdetprt_resol = vdet[hit.idet] - vhit_resol - vreac_resol; */
      
      /* E.lab2_det_resol = ran->Gaus(E.lab2_det, // based on resolution at 5 MeV */
      /* 			    sqrt(5.*E.lab2_det)*depe2); // MeV */
      /* // Ecm_resol_nr = Elab2cm(Elab2_resol, thlab2_resol, mres[iex], q2[iex]); // ! */
      /* // thcm_resol_nr = thlab2cm(Ecm_resol,thlab2_resol, mres[iex], q2[iex]); // ! */
      
      /* // Newton's method */
      /* const Int_t nlim = 10; */
      /* const Double_t dztar = .2; */
      /* const Double_t eps = 1.e-4; */
      /* TVector3 vreac_resol; */
      /* float fldetprt_resol, z_beam_resol; */
      /* pos.ztar_resol = pos.ztar; */
      /* Ebfrpu_beam = eprj0ini/mprj*1000.; */
      /* Eaftpu = E.lab2_det_resol/mrec; // MeV */
      /* Double_t eprj0_resol = eprj0; // initial value */
      /* //cout << eprj0_resol*1000. << endl; */
      /* Double_t Ecm_resol_beam, Ecm_resol_beam_p, Ecm_resol_beam_m, */
      /*   Ecm_resol_rec, Ecm_resol_rec_p, Ecm_resol_rec_m; */
      /* //cout << "2" << endl; getchar(); */
      /* if(!isnan(Elab2cm(E.lab[1], th.lab[1], mres[iex], q2[iex]))){ */
      /* //   cout << E.cm << " " << Elab2cm(E.lab[1], th.lab[1], mres[iex], q2[iex]) << endl; getchar(); */
      /* //   E.beam0_resol = E.beam0; */
      /* //   E.cm_z_resol = E.cm_z; */
      /* //   E.cm_resol = E.cm; */
      /* //   pos.ldetprt_resol = pos.ldetprt; */
      /* //   phi.lab2_resol = phi.lab[1]; */
      /* // }else{ */
      /*   for(Int_t i=0; i<nlim; i++){ // no. of trials for Newton's method */
      /*     // Ecm from recoil energy at z */
      /*     Ecm_resol_beam = eprj0_resol*mtrg/(mprj+mtrg)*1000.; // Ecm from beam energy */
      
      /*     vreac_resol.SetXYZ(pos.xtar_resol,pos.ytar_resol,pos.ztar_resol); */
      /*     vdetprt_resol = vhit_resol + vdet[hit.idet] - vreac_resol; */
      /*     fldetprt_resol = float(vdetprt_resol.Mag()); */
      /*     eoldzsub_(&zrec, &mrec, &Eaftpu, he, &unit_pressure, &pressure0,  */
      /* 	      &temperature0, &unit_mm, &fldetprt_resol, &Ebfrpu); */
      /*     E.lab2_resol = Ebfrpu*mrec; // MeV	   */
      /*     th.lab2_resol = (vdetprt_resol.Theta())/d2r; */
      /*     Ecm_resol_rec = Elab2cm(E.lab2_resol, th.lab2_resol, mres[iex], q2[iex]); */
      
      /*     // cout << i << " " << eprj0_resol*1000. << " " << ztar_resol */
      /*     //      << " " << Ecm_resol_beam << " " << Ecm_resol_rec << endl; */
      
      /*     if(2.*fabs(Ecm_resol_beam - Ecm_resol_rec)/(Ecm_resol_beam + Ecm_resol_rec) < eps){ */
      /*       //cout << 2.*fabs(Ecm_resol_beam - Ecm_resol_rec)/(Ecm_resol_beam + Ecm_resol_rec) << endl; */
      /*       break; */
      /*     } */
      /*     // Ecm from recoil energy at z+dz */
      /*     z_beam_resol = pos.ztar_resol+dztar + ltar/2.; */
      /*     enewzsub_(&zprj, &mprj, &Ebfrpu_beam, he, &unit_pressure, &pressure0,  */
      /* 	      &temperature0, &unit_mm, &z_beam_resol, &Eaftpu_beam); */
      /*     //cout << i << " " << Ebfrpu_beam << " " << z_beam_resol << " " << Eaftpu_beam; */
      /*     Ecm_resol_beam_p = Eaftpu_beam*mprj*mtrg/(mprj+mtrg); */
      
      /*     vreac_resol.SetXYZ(pos.xtar_resol,pos.ytar_resol,pos.ztar_resol+dztar); */
      /*     vdetprt_resol = vhit_resol + vdet[hit.idet] - vreac_resol; */
      /*     fldetprt_resol = float(vdetprt_resol.Mag()); */
      /*     eoldzsub_(&zrec, &mrec, &Eaftpu, he, &unit_pressure, &pressure0,  */
      /* 	      &temperature0, &unit_mm, &fldetprt_resol, &Ebfrpu); */
      /*     E.lab2_resol = Ebfrpu*mrec; // MeV	   */
      /*     th.lab2_resol = (vdetprt_resol.Theta())/d2r; */
      /*     Ecm_resol_rec_p = Elab2cm(E.lab2_resol, th.lab2_resol, mres[iex], q2[iex]); */
      
      /*     // Ecm from recoil energy at z-dz */
      /*     z_beam_resol = pos.ztar_resol-dztar + ltar/2.; */
      /*     enewzsub_(&zprj, &mprj, &Ebfrpu_beam, he, &unit_pressure, &pressure0,  */
      /* 	      &temperature0, &unit_mm, &z_beam_resol, &Eaftpu_beam); */
      /*     //cout << " " << Ebfrpu_beam << " " << z_beam_resol << " " << Eaftpu_beam; // << endl; */
      /*     Ecm_resol_beam_m = Eaftpu_beam*mprj*mtrg/(mprj+mtrg); */
      
      /*     vreac_resol.SetXYZ(pos.xtar_resol,pos.ytar_resol,pos.ztar_resol-dztar); */
      /*     vdetprt_resol = vhit_resol + vdet[hit.idet] - vreac_resol; */
      /*     fldetprt_resol = float(vdetprt_resol.Mag()); */
      /*     eoldzsub_(&zrec, &mrec, &Eaftpu, he, &unit_pressure, &pressure0,  */
      /* 	      &temperature0, &unit_mm, &fldetprt_resol, &Ebfrpu); */
      /*     E.lab2_resol = Ebfrpu*mrec; // MeV	   */
      /*     th.lab2_resol = (vdetprt_resol.Theta())/d2r; */
      /*     Ecm_resol_rec_m = Elab2cm(E.lab2_resol, th.lab2_resol, mres[iex], q2[iex]); */
      
      /*     // differential (f(z) = E_beam - E_rec) */
      /*     Double_t dEcmdz = (Ecm_resol_beam_p - Ecm_resol_rec_p */
      /* 		       - Ecm_resol_beam_m + Ecm_resol_rec_m) */
      /*       /2./dztar; */
      /*     // cout << i  */
      /*     //      << " " << pos.ztar_resol */
      /*     //      << " " << fldetprt_resol */
      /*     //      << " " << Eaftpu*mrec      << " " << E.lab2_resol */
      /*     //      << " " << th.lab2_resol */
      /*     //      << " " << Ecm_resol_beam_p << " " << Ecm_resol_beam_m */
      /*     //      << " " << Ecm_resol_rec_p  << " " << Ecm_resol_rec_m */
      /*     //      << " " << dEcmdz */
      /*     //      << " " << (Ecm_resol_beam - Ecm_resol_rec)/dEcmdz */
      /*     //      << " " << 2.*fabs(Ecm_resol_beam - Ecm_resol_rec)/(Ecm_resol_beam + Ecm_resol_rec) */
      /*     //      << endl; getchar(); */
      /*     pos.ztar_resol -= (Ecm_resol_beam - Ecm_resol_rec)/dEcmdz;	  	   */
      
      /*     // For Ebeam at next ztar_resol */
      /*     z_beam_resol = pos.ztar_resol + ltar/2.; */
      /*     enewzsub_(&zprj, &mprj, &Ebfrpu_beam, he, &unit_pressure, &pressure0,  */
      /* 	      &temperature0, &unit_mm, &z_beam_resol, &Eaftpu_beam); */
      /*     eprj0_resol = Eaftpu_beam*mprj/1000.; */
      /*     // cout << " " << Ebfrpu_beam << " " << z_beam_resol << " " << Eaftpu_beam */
      /*     //      << " " << dEcmdz << endl; */
      /*   }	 */
      /*   E.beam0_resol = eprj0_resol*1000.; */
      /*   E.cm_z_resol = eprj0_resol*mtrg/(mprj+mtrg)*1000.; */
      /*   E.cm_resol = (Ecm_resol_beam + Ecm_resol_rec)/2.; */
      /*   pos.ldetprt_resol = vdetprt_resol.Mag(); */
      /*   phi.lab2_resol = (vdetprt_resol.Phi())/d2r; */
      /* } */
      
      /* TLorentzVector beam_resol(0.0, 0.0, sqrt(2.*u*mprj*eprj0_resol), u*mprj+eprj0_resol); // px, py, pz, e */
      /* TLorentzVector W_resol = beam_resol + target; // compound of beam + target */
      /* TLorentzVector p_resol[2]; */
      /* TVector3 dir_resol = vdetprt_resol.Unit(); */
      /* p_resol[1].SetPxPyPzE(sqrt(2.*mass[1]*E.lab2_resol/1000.)*dir_resol.X(),  */
      /* 		      sqrt(2.*mass[1]*E.lab2_resol/1000.)*dir_resol.Y(),  */
      /* 		      sqrt(2.*mass[1]*E.lab2_resol/1000.)*dir_resol.Z(),  */
      /* 		      mass[1]+E.lab2_resol/1000.); */
      /* p_resol[0] = W_resol - p_resol[1];	 */
      /* TVector3 vprj_resol = pow(u*mprj,-1)/beam_resol.Gamma() *(beam_resol.Vect()); // velocity  */
      /* TVector3 v1_resol = pow(u*mres[iex],-1)/p_resol[0].Gamma() *(p_resol[0].Vect()); */
      /* TVector3 v2_resol = pow(u*mrec,-1)/p_resol[1].Gamma() *(p_resol[1].Vect()); */
      /* TVector3 ki_resol = vprj_resol;  */
      /* TVector3 kf_resol = v2_resol - v1_resol;  */
      /* // TLorentzVector p12_resol = p_resol[0] + p_resol[1]; //  */
      /* // Double_t e12_resol = (p12_resol.M() - u*(mres[iex] + mrec)); //Erel = e12*1000.; //val[9] = e12; // MeV */
      /* //Ecm_resol = (e12_resol - q2[iex])*1000.; // MeV */
      /* th.cm_resol = (ki_resol.Angle(kf_resol))/d2r; */
      /* //cout << id << " " << Ecm << " " << Ecm_resol << " " << thcm << " " << thcm_resol << endl; */
      /* //cout << "3" << endl; //getchar(); */
      /* // cout << "ztar: " << ztar << " " << ztar_resol << endl; */
      /* // cout << "ldetprt: " << ldetprt << " " << ldetprt_resol << endl; */
      /* // cout << "Ebeam: " << Ebeam << " " << Ebeam0_resol << endl; */
      /* // cout << "Ecm_z: " << Ecm_z << " " << Ecm_z_resol << endl; */
      /* // cout << "Ecm: " << Ecm << " " << Ecm_resol << endl; */
      /* // cout << "thlab2: " << thlab[1] << " " << thlab2_resol << endl; */
      /* // cout << "philab2: " << philab[1] << " " << philab2_resol << endl; */
      /* // cout << "thcm: " << thcm << " " << thcm_resol << endl; */
      /* // cout << endl; getchar(); */
      
      /* E.cm_dev = E.cm - E.cm_resol; */
      /* th.cm_dev = th.cm - th.cm_resol; */
      
      /* if( E.cm>2. && E.lab2_det>0. && E.lab2_det_resol>0. */
      /*    && !isnan(z_beam) && !isnan(z_beam_resol) */
      /*    && !isnan(pos.ldetprt) && !isnan(pos.ldetprt_resol) ){ */
      /*   //if(E.cm>1.){ */
      /*   // TOF calucluation */
      /*   Ebfrpu_beam = eprjini/mprj*1000.; // beam energy at entrance, MeV  */
      /*   fint->SetParameters(zprj,mprj,Ebfrpu_beam); */
      /*   tof.beam = fint->Integral(0.,z_beam,1e-4);	   */
      /*   // if(isnan(tof.beam)){ */
      /*   //   cout << "Beam: " << Ebfrpu_beam <<" " << z_beam << " " << tof.beam << endl; */
      /*   //   getchar(); */
      /*   // } */
      
      /*   // TOF calucluation, with resolution */
      /*   //float z_beam_resol = ztar_resol + ltar/2.; */
      /*   //Eaftpu_beam = Ebfrpu_beam; // beam energy at reaction point, MeV */
      /*   // Eaftpu_beam = Ebeam0_resol/mprj; // beam energy at reaction point, MeV */
      /*   // eoldzsub_(&zprj, &mprj, &Eaftpu_beam, he, &unit_pressure, &pressure,  */
      /*   // 	    &temperature, &unit_mm, &z_beam_resol, &Ebfrpu_beam); */
      /*   Ebfrpu_beam = eprj0ini/mprj*1000.; // beam energy at entrance, MeV  */
      /*   fint->SetParameters(zprj,mprj,Ebfrpu_beam); */
      /*   tof.beam_resol = fint->Integral(0.,z_beam_resol,1e-4); */
      /*   // if(isnan(tof.beam_resol)){ */
      /*   //   cout << "Beam resol: " << Ebfrpu_beam <<" " << z_beam_resol << " " << tof.beam_resol << endl; */
      /*   //   getchar(); */
      /*   // } */
      
      /*   // TOF calucluation */
      /*   Ebfrpu = E.lab[1]/mrec; */
      /*   fint->SetParameters(zrec,mrec,Ebfrpu); */
      /*   tof.rec = fint->Integral(0.,pos.ldetprt,1e-4); */
      /*   // if(isnan(tof.rec)){ */
      /*   //   cout << "Rec: " << Ebfrpu <<" " << pos.ldetprt << " " << tof.rec << endl; */
      /*   //   getchar(); */
      /*   // } */
      
      /*   // TOF calucluation, with resolution */
      /*   //Ebfrpu = Elab[1]/mrec; */
      /*   Ebfrpu = E.lab2_resol/mrec; */
	/*   fint->SetParameters(zrec,mrec,Ebfrpu); */
	/*   //float fldetprt_resol = float(ldetprt_resol); */
	/*   tof.rec_resol = fint->Integral(0.,pos.ldetprt_resol,1e-4); */
	/*   // if(isnan(tof.rec_resol)){ */
	/*   //   cout << "Rec resol: " << Ebfrpu <<" " << pos.ldetprt_resol << " " << tof.rec_resol << endl; */
	/*   //   getchar(); */
	/*   // } */
	  
	/*   // DE layer */
	/*   Ebfrpu = E.lab2_det/mrec; // Energy in front of DE (MeV) */
	/*   enewzsub_(&zrec, &mrec, &Ebfrpu, si, &unit_pressure, &pressure,  */
	/* 	    &temperature, &unit_mm, &de_thick, &Eaftpu); */
	/*   E.lab2_DE = (Ebfrpu - Eaftpu)*mrec; // Energy in DE */
	/*   //DElab2_resol = ran->Gaus(DElab2,DElab2*depe2); */
	/*   E.lab2_DE_resol = ran->Gaus(E.lab2_DE, // based on resolution at 5 MeV */
	/* 			     sqrt(5.*E.lab2_DE)*depe2); */
	  
	/*   Ebfrpu = Eaftpu; // Energy after DE */
	/*   enewzsub_(&zrec, &mrec, &Ebfrpu, he, &unit_pressure, &pressure,  */
	/* 	  &temperature, &unit_mm, &int_det, &Eaftpu);	 */
	/*   E.lab2_RES = Eaftpu*mrec; // Energy before E */
	/*   E.lab2_RES_resol = ran->Gaus(E.lab2_RES, // based on resolution at 5 MeV */
	/* 			   sqrt(5.*E.lab2_RES)*depe2); */
	/*   //cout << "5" << endl; getchar(); */
	/*   // if(Eaftpu > 0.){ */
	/*   //   fint->SetParameters(zrec,mrec,Ebfrpu); */
	/*   //   tofdee = fint->Integral(0.,int_det,1e-3); */
	/*   // } */
	/*   tof.de = tof.beam + tof.rec; // TOF up to DE layer */
	/*   tof.de_resol = ran->Gaus(tof.beam_resol + tof.rec_resol,dt); */
	  // tofe = tofde + tofdee; // TOF up to E layer
	  // tofe_resol = ran->Gaus(tofe,dt);
      //}// if(Elab2_det>0)
	
    } // if istate2
    geve++;
    //} // if eprj>0
    
    // w.efth = eff_thk(eprj);
    // if(E.cm_z < 0.36686) // Min. energy of NONSMOKER calc.
    //   w.hf = 0.;
    // else
    //   w.hf = interhf.Eval(E.cm_z);
    // w.yld = Ccs*w.hf; 
    //cout << w.efth << " " << w.hf << " " << w.yld << endl;
    
    //Double_t rnum = ran->Rndm();
    //if(rnum<=eff_thk(ps)){
    sim->Fill();	  
    //geve++;
    //}
    
    //    } // if istate2
    //i++;
    //cout << geve << "\n"; 
  } // for 
  fl2->Write();
  //fl2->Close();
  cout << "Number of events inside the detectors =" << geve << "\n"; 
  cout << "Total number of events =" << ntot << "\n";
  cout << "Done!" << ntot << "\n"; 
  //return 0;

  TView3D *view = (TView3D*)TView::CreateView(1);
  view->SetRange(-250,-250,-250,250,250,250);
  view->Front();
  view->ShowAxis();  
  //c0->Update();gSystem->ProcessEvents();getchar();
  //c0->Print("detector_array.png");
  //c0->Print("detector_array_sim01.png");
  //c0->Print("detector_array_sim01.png");
  
  theApp.Run();
  
  return 0;
  
}


    /* //pdir[0] = pmom[0].Unit(); // unit vector of the momentum */
    /* pdir[1] = pmom[1].Unit(); // represents the direction */
    /* xx[1] = pdir[1](0); */
    /* yy[1] = pdir[1](1); */
    /* zz[1] = pdir[1](2); */
    /* gGeoManager->SetCurrentPoint(pos.xtar,pos.ytar,pos.ztar); */
    /* gGeoManager->SetCurrentDirection(xx[1],yy[1],zz[1]); */
    /* //      TGeoNode *current2 = */
    /* gGeoManager->GetCurrentNode(); */
    /* gGeoManager->FindNode(); */
    /* gGeoManager->FindNextBoundary(radius); */
    /* snext[1] = gGeoManager->GetStep(); */
    /* //TGeoNode *newNode2 = */
    /* gGeoManager->Step(); */
    /* istate[1] = gGeoManager->IsStepEntering(); */
    /* idnode[1] = gGeoManager->GetCurrentNodeId(); */
    /* //Double_t safety2 = gGeoManager->GetSafeDistance(); */
    /* //cout << istate2 <<" "<< snext2 << endl; */
    /* path = gGeoManager->GetPath(); // /TOP_1/PSD_3 etc. */
    /* strncpy(sdet, path+11, 1); // extract the 12th character from path */
    /* sdet[1] = '\0'; */
    /* hitdet[1] = atoi(sdet); */
    /* //TVector3 vdet2(snext2*xx2,snext2*yy2,snext2*zz2);  */
    /* vdetprt[1].SetXYZ(snext[1]*xx[1],snext[1]*yy[1],snext[1]*zz[1]); */
    /* pos.ldetprt[1] = snext[1]; */
    
    // E.beam = E.beam0 = E.beam0_resol
    //   = E.lab[0] = E.lab[1] = E.lab[2]
    //   = E.lab_det[0] = E.lab_det[1]
    //   = E.lab_det_resol[0] = E.lab_det_resol[1]
    //   = E.lab_resol[0] = E.lab_resol[1]
    //   = dummy;
    // E.lab_DE[0] = E.lab_DE[1]
    //   = E.lab_DE_resol[0] = E.lab_DE_resol[1]
    //   = E.lab_RES[0] = E.lab_RES[1]
    //   = E.lab_RES_resol[0] = E.lab_RES_resol[1]
    //   = dummy;
    // th.lab[0] = th.lab[1] = th.lab[2]
    //   = phi.lab[0] = phi.lab[1] = phi.lab[2]
    //   = th.lab_resol[0] = th.lab_resol[1]
    //   = phi.lab_resol[0] = phi.lab_resol[1]
    //   = th.cm = th.cm_resol = th.cm_dev = 0.;
    // th.cm_resol_nr = 0.;

    // E.rel[0] = E.rel[1] = E.rel[2]
    //   = E.rel_resol[0] = E.rel_resol[1] = E.rel_resol[2]
    //   = E.cm = E.cm_z = E.cm_resol = E.cm_z_resol= E.cm_z0 = E.cm_dev
    //   = dummy; 
    //p.spc = dummy;

    // tof.beam = tof.beam_resol
    //   = tof.rec = tof.rec_resol
    //   = tof.de = tof.de_resol = dummy;
    //tof.dee = tof.e= tof.e_resol = dummy;a
    // for(Int_t i=0;i<ndet;i++){
    //   pos.xhit[0][i] = pos.yhit[0][i] = pos.xhit_resol[0][i] = pos.yhit_resol[0][i] = dummy;
    //   pos.xhit[1][i] = pos.yhit[1][i] = pos.xhit_resol[1][i] = pos.yhit_resol[1][i] = dummy;
    // }
    // for(Int_t i=0;i<ndet;i++)
    //   tel[i] = false; // initialization

    //hit.any = 0; // initialization
    // pos.xtar = pos.ytar = pos.rtar
    //   = pos.xtar_resol = pos.ytar_resol //= rtar_resol
    //   = pos.ldetprt[0] = pos.ldetprt[1]
    //   = pos.ldetprt_resol[0] = pos.ldetprt_resol[1] = dummy;

    //w.evt = 0.; //w.efth = w.hf = w.yld = 0.; 


    /* //-- Defining the reaction point --//     */
    /* // nran = ran->Rndm(); // 0-1 */
    /* // ztar = ltar*(nran-0.5); // 0 +/- ltar/2  // ! */
    /* // Uniform in Ebeam */
    /* eprjini = ran->Gaus(eprj0ini,deprj); // ! for now deprj is constant */
    /* eprj = ran->Uniform(0.,eprjini); */
    /* float z_beam = ebeam2thk(eprj*1000.,eprjini*1000.)/density/1000.*10.; */
    /* //cout << eprjini << " " << eprj << " " << z_beam << endl; */
    
    /* // pos.ztar = ran->Uniform(-ltar/2.,ltar/2.); // Reaction point, 10-nov-2021 */
    /* //float z_beam = pos.ztar + ltar/2.; // 0-ltar for enewz */
    /* pos.ztar = z_beam - ltar/2.; */
    /* //cout << pos.ztar << endl; */
    /* Ebfrpu_beam = eprj0ini/mprj*1000.; // MeV */
    /* enewzsub_(&zprj, &mprj, &Ebfrpu_beam, he, &unit_pressure, &pressure0, */
    /* 	      &temperature0, &unit_mm, &z_beam, &Eaftpu_beam); */
    /* eprj0 = Eaftpu_beam*mprj/1000.; // Mean beam energy at reaction point, GeV */
    /* E.beam0 = eprj0*1000.; // MeV */
    /* E.cm_z0 = eprj0*mtrg/(mprj+mtrg)*1000.; // MeV */
    
    /* pressure = ran->Gaus(pressure0,dpressure); */
    /* temperature = ran->Gaus(temperature0,dtemperature); */
    /* gas.prs0 = double(pressure0); */
    /* gas.prs = double(pressure); */
    /* gas.tmp0 = double(temperature0); */
    /* gas.tmp = double(temperature); */

    /* // eprjini = ran->Gaus(eprj0ini,deprj); // ! for now deprj is constant */
    /* // Ebfrpu_beam = eprjini/mprj*1000.; // MeV */
    /* // enewzsub_(&zprj, &mprj, &Ebfrpu_beam, he, &unit_pressure, &pressure, */
    /* // 	      &temperature, &unit_mm, &z_beam, &Eaftpu_beam); */
    /* //    eprj = Eaftpu_beam*mprj/1000.; // Deviated beam energy at reaction point, GeV */
    /* E.beam = eprj*1000.; // MeV */
    /* E.cm_z = eprj*mtrg/(mprj+mtrg)*1000.; // MeV */
    // cout << pos.ztar << " " << E.beam0 << " " << E.beam
    // 	 << " " << E.cm_z0 << " " << E.cm_z << endl;

